/****************************************************************************
 * Copyright (C) 2016 Intel Corporation.   All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * @file gen_ar_event.hpp
 *
 * @brief Definitions for events.  auto-generated file
 *
 * DO NOT EDIT
 *
 * Generation Command Line:
 *  ../src/gallium/drivers/swr/rasterizer/codegen/gen_archrast.py
 *    --proto
 *    ../src/gallium/drivers/swr/rasterizer/archrast/events.proto
 *    ../src/gallium/drivers/swr/rasterizer/archrast/events_private.proto
 *    --output-dir
 *    /home/iurt/rpmbuild/BUILD/mesa-20.3.4/build/src/gallium/drivers/swr/rasterizer/codegen
 *
 ******************************************************************************/
// clang-format off
#pragma once

#include "common/os.h"
#include "core/state.h"


namespace ArchRast
{

    enum AR_DRAW_TYPE
    {
        Instanced = 0,
        IndexedInstanced = 1,
        InstancedSplit = 2,
        IndexedInstancedSplit = 3
    };

    // Forward decl
    class EventHandler;

    //////////////////////////////////////////////////////////////////////////
    /// Event - interface for handling events.
    //////////////////////////////////////////////////////////////////////////
    struct Event
    {
        const uint32_t eventId = {0xFFFFFFFF};
        Event() {}
        virtual ~Event() {}

        virtual bool IsEnabled() const { return true; };
        virtual const uint32_t GetEventId() const = 0;
        virtual void Accept(EventHandler* pHandler) const = 0;
    };



    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::FrameEndEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct FrameEndEventData
    {
        // Fields
        uint32_t frameId;
        uint32_t nextDrawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::FrameEndEvent
    //////////////////////////////////////////////////////////////////////////
    struct FrameEndEvent : Event
    {
        const uint32_t eventId = {3};
        FrameEndEventData data;

        // Constructor
        FrameEndEvent(
            uint32_t frameId,
            uint32_t nextDrawId
        )
        {
            data.frameId = frameId;
            data.nextDrawId = nextDrawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrDiscardRectEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct SwrDiscardRectEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrDiscardRectEvent
    //////////////////////////////////////////////////////////////////////////
    struct SwrDiscardRectEvent : Event
    {
        const uint32_t eventId = {6};
        SwrDiscardRectEventData data;

        // Constructor
        SwrDiscardRectEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrInvalidateTilesEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct SwrInvalidateTilesEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrInvalidateTilesEvent
    //////////////////////////////////////////////////////////////////////////
    struct SwrInvalidateTilesEvent : Event
    {
        const uint32_t eventId = {5};
        SwrInvalidateTilesEventData data;

        // Constructor
        SwrInvalidateTilesEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrStoreTilesEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct SwrStoreTilesEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrStoreTilesEvent
    //////////////////////////////////////////////////////////////////////////
    struct SwrStoreTilesEvent : Event
    {
        const uint32_t eventId = {7};
        SwrStoreTilesEventData data;

        // Constructor
        SwrStoreTilesEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrSyncEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct SwrSyncEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ApiSwr::SwrSyncEvent
    //////////////////////////////////////////////////////////////////////////
    struct SwrSyncEvent : Event
    {
        const uint32_t eventId = {4};
        SwrSyncEventData data;

        // Constructor
        SwrSyncEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };



    //////////////////////////////////////////////////////////////////////////
    /// Framework::ThreadStartApiEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct ThreadStartApiEventData
    {
        // Fields
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// Framework::ThreadStartApiEvent
    //////////////////////////////////////////////////////////////////////////
    struct ThreadStartApiEvent : Event
    {
        const uint32_t eventId = {1};
        ThreadStartApiEventData data;

        // Constructor
        ThreadStartApiEvent(
        )
        {
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };


    //////////////////////////////////////////////////////////////////////////
    /// Framework::ThreadStartWorkerEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct ThreadStartWorkerEventData
    {
        // Fields
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// Framework::ThreadStartWorkerEvent
    //////////////////////////////////////////////////////////////////////////
    struct ThreadStartWorkerEvent : Event
    {
        const uint32_t eventId = {2};
        ThreadStartWorkerEventData data;

        // Constructor
        ThreadStartWorkerEvent(
        )
        {
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
    };



    //////////////////////////////////////////////////////////////////////////
    /// Memory::MemoryAccessEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct MemoryAccessEventData
    {
        // Fields
        uint32_t drawId;
        uint64_t tsc;
        uint64_t ptr;
        uint32_t size;
        uint8_t isRead;
        uint8_t client;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// Memory::MemoryAccessEvent
    //////////////////////////////////////////////////////////////////////////
    struct MemoryAccessEvent : Event
    {
        const uint32_t eventId = {56};
        MemoryAccessEventData data;

        // Constructor
        MemoryAccessEvent(
            uint32_t drawId,
            uint64_t tsc,
            uint64_t ptr,
            uint32_t size,
            uint8_t isRead,
            uint8_t client
        )
        {
            data.drawId = drawId;
            data.tsc = tsc;
            data.ptr = ptr;
            data.size = size;
            data.isRead = isRead;
            data.client = client;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_MEMORY_EVENTS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// Memory::MemoryStatsEndEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct MemoryStatsEndEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// Memory::MemoryStatsEndEvent
    //////////////////////////////////////////////////////////////////////////
    struct MemoryStatsEndEvent : Event
    {
        const uint32_t eventId = {57};
        MemoryStatsEndEventData data;

        // Constructor
        MemoryStatsEndEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_MEMORY_EVENTS && IsEventEnabled;
        }
    };



    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::AlphaEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct AlphaEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t alphaTestCount;
        uint32_t alphaBlendCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::AlphaEvent
    //////////////////////////////////////////////////////////////////////////
    struct AlphaEvent : Event
    {
        const uint32_t eventId = {39};
        AlphaEventData data;

        // Constructor
        AlphaEvent(
            uint32_t drawId,
            uint32_t alphaTestCount,
            uint32_t alphaBlendCount
        )
        {
            data.drawId = drawId;
            data.alphaTestCount = alphaTestCount;
            data.alphaBlendCount = alphaBlendCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::AlphaInfoEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct AlphaInfoEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t alphaTestEnable;
        uint32_t alphaBlendEnable;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::AlphaInfoEvent
    //////////////////////////////////////////////////////////////////////////
    struct AlphaInfoEvent : Event
    {
        const uint32_t eventId = {63};
        AlphaInfoEventData data;

        // Constructor
        AlphaInfoEvent(
            uint32_t drawId,
            uint32_t alphaTestEnable,
            uint32_t alphaBlendEnable
        )
        {
            data.drawId = drawId;
            data.alphaTestEnable = alphaTestEnable;
            data.alphaBlendEnable = alphaBlendEnable;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::BackendDrawEndEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct BackendDrawEndEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::BackendDrawEndEvent
    //////////////////////////////////////////////////////////////////////////
    struct BackendDrawEndEvent : Event
    {
        const uint32_t eventId = {54};
        BackendDrawEndEventData data;

        // Constructor
        BackendDrawEndEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::BackendStatsEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct BackendStatsEventData
    {
        // Fields
        uint32_t drawId;
        uint64_t DepthPassCount;
        uint64_t PsInvocations;
        uint64_t CsInvocations;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::BackendStatsEvent
    //////////////////////////////////////////////////////////////////////////
    struct BackendStatsEvent : Event
    {
        const uint32_t eventId = {11};
        BackendStatsEventData data;

        // Constructor
        BackendStatsEvent(
            uint32_t drawId,
            uint64_t DepthPassCount,
            uint64_t PsInvocations,
            uint64_t CsInvocations
        )
        {
            data.drawId = drawId;
            data.DepthPassCount = DepthPassCount;
            data.PsInvocations = PsInvocations;
            data.CsInvocations = CsInvocations;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::ClipInfoEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct ClipInfoEventData
    {
        // Fields
        uint32_t numInvocations;
        uint32_t validMask;
        uint32_t clipMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::ClipInfoEvent
    //////////////////////////////////////////////////////////////////////////
    struct ClipInfoEvent : Event
    {
        const uint32_t eventId = {61};
        ClipInfoEventData data;

        // Constructor
        ClipInfoEvent(
            uint32_t numInvocations,
            uint32_t validMask,
            uint32_t clipMask
        )
        {
            data.numInvocations = numInvocations;
            data.validMask = validMask;
            data.clipMask = clipMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::ClipperEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct ClipperEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t trivialRejectCount;
        uint32_t trivialAcceptCount;
        uint32_t mustClipCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::ClipperEvent
    //////////////////////////////////////////////////////////////////////////
    struct ClipperEvent : Event
    {
        const uint32_t eventId = {37};
        ClipperEventData data;

        // Constructor
        ClipperEvent(
            uint32_t drawId,
            uint32_t trivialRejectCount,
            uint32_t trivialAcceptCount,
            uint32_t mustClipCount
        )
        {
            data.drawId = drawId;
            data.trivialRejectCount = trivialRejectCount;
            data.trivialAcceptCount = trivialAcceptCount;
            data.mustClipCount = mustClipCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::CullEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct CullEventData
    {
        // Fields
        uint32_t drawId;
        uint64_t backfacePrimCount;
        uint64_t degeneratePrimCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::CullEvent
    //////////////////////////////////////////////////////////////////////////
    struct CullEvent : Event
    {
        const uint32_t eventId = {38};
        CullEventData data;

        // Constructor
        CullEvent(
            uint32_t drawId,
            uint64_t backfacePrimCount,
            uint64_t degeneratePrimCount
        )
        {
            data.drawId = drawId;
            data.backfacePrimCount = backfacePrimCount;
            data.degeneratePrimCount = degeneratePrimCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::CullInfoEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct CullInfoEventData
    {
        // Fields
        uint32_t drawId;
        uint64_t degeneratePrimMask;
        uint64_t backfacePrimMask;
        uint32_t validMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::CullInfoEvent
    //////////////////////////////////////////////////////////////////////////
    struct CullInfoEvent : Event
    {
        const uint32_t eventId = {62};
        CullInfoEventData data;

        // Constructor
        CullInfoEvent(
            uint32_t drawId,
            uint64_t degeneratePrimMask,
            uint64_t backfacePrimMask,
            uint32_t validMask
        )
        {
            data.drawId = drawId;
            data.degeneratePrimMask = degeneratePrimMask;
            data.backfacePrimMask = backfacePrimMask;
            data.validMask = validMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DispatchEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DispatchEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t threadGroupCountX;
        uint32_t threadGroupCountY;
        uint32_t threadGroupCountZ;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DispatchEvent
    //////////////////////////////////////////////////////////////////////////
    struct DispatchEvent : Event
    {
        const uint32_t eventId = {9};
        DispatchEventData data;

        // Constructor
        DispatchEvent(
            uint32_t drawId,
            uint32_t threadGroupCountX,
            uint32_t threadGroupCountY,
            uint32_t threadGroupCountZ
        )
        {
            data.drawId = drawId;
            data.threadGroupCountX = threadGroupCountX;
            data.threadGroupCountY = threadGroupCountY;
            data.threadGroupCountZ = threadGroupCountZ;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawIndexedInstancedEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DrawIndexedInstancedEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t topology;
        uint32_t numIndices;
        int32_t indexOffset;
        int32_t baseVertex;
        uint32_t numInstances;
        uint32_t startInstance;
        uint32_t tsEnable;
        uint32_t gsEnable;
        uint32_t soEnable;
        uint32_t soTopology;
        uint32_t splitId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawIndexedInstancedEvent
    //////////////////////////////////////////////////////////////////////////
    struct DrawIndexedInstancedEvent : Event
    {
        const uint32_t eventId = {65};
        DrawIndexedInstancedEventData data;

        // Constructor
        DrawIndexedInstancedEvent(
            uint32_t drawId,
            uint32_t topology,
            uint32_t numIndices,
            int32_t indexOffset,
            int32_t baseVertex,
            uint32_t numInstances,
            uint32_t startInstance,
            uint32_t tsEnable,
            uint32_t gsEnable,
            uint32_t soEnable,
            uint32_t soTopology,
            uint32_t splitId
        )
        {
            data.drawId = drawId;
            data.topology = topology;
            data.numIndices = numIndices;
            data.indexOffset = indexOffset;
            data.baseVertex = baseVertex;
            data.numInstances = numInstances;
            data.startInstance = startInstance;
            data.tsEnable = tsEnable;
            data.gsEnable = gsEnable;
            data.soEnable = soEnable;
            data.soTopology = soTopology;
            data.splitId = splitId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawInfoEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DrawInfoEventData
    {
        // Fields
        uint32_t drawId;
        AR_DRAW_TYPE type;
        uint32_t topology;
        uint32_t numVertices;
        uint32_t numIndices;
        int32_t indexOffset;
        int32_t baseVertex;
        uint32_t numInstances;
        uint32_t startInstance;
        uint32_t tsEnable;
        uint32_t gsEnable;
        uint32_t soEnable;
        uint32_t soTopology;
        uint32_t splitId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawInfoEvent
    //////////////////////////////////////////////////////////////////////////
    struct DrawInfoEvent : Event
    {
        const uint32_t eventId = {8};
        DrawInfoEventData data;

        // Constructor
        DrawInfoEvent(
            uint32_t drawId,
            AR_DRAW_TYPE type,
            uint32_t topology,
            uint32_t numVertices,
            uint32_t numIndices,
            int32_t indexOffset,
            int32_t baseVertex,
            uint32_t numInstances,
            uint32_t startInstance,
            uint32_t tsEnable,
            uint32_t gsEnable,
            uint32_t soEnable,
            uint32_t soTopology,
            uint32_t splitId
        )
        {
            data.drawId = drawId;
            data.type = type;
            data.topology = topology;
            data.numVertices = numVertices;
            data.numIndices = numIndices;
            data.indexOffset = indexOffset;
            data.baseVertex = baseVertex;
            data.numInstances = numInstances;
            data.startInstance = startInstance;
            data.tsEnable = tsEnable;
            data.gsEnable = gsEnable;
            data.soEnable = soEnable;
            data.soTopology = soTopology;
            data.splitId = splitId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawInstancedEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DrawInstancedEventData
    {
        // Fields
        uint32_t drawId;
        uint32_t topology;
        uint32_t numVertices;
        int32_t startVertex;
        uint32_t numInstances;
        uint32_t startInstance;
        uint32_t tsEnable;
        uint32_t gsEnable;
        uint32_t soEnable;
        uint32_t soTopology;
        uint32_t splitId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::DrawInstancedEvent
    //////////////////////////////////////////////////////////////////////////
    struct DrawInstancedEvent : Event
    {
        const uint32_t eventId = {64};
        DrawInstancedEventData data;

        // Constructor
        DrawInstancedEvent(
            uint32_t drawId,
            uint32_t topology,
            uint32_t numVertices,
            int32_t startVertex,
            uint32_t numInstances,
            uint32_t startInstance,
            uint32_t tsEnable,
            uint32_t gsEnable,
            uint32_t soEnable,
            uint32_t soTopology,
            uint32_t splitId
        )
        {
            data.drawId = drawId;
            data.topology = topology;
            data.numVertices = numVertices;
            data.startVertex = startVertex;
            data.numInstances = numInstances;
            data.startInstance = startInstance;
            data.tsEnable = tsEnable;
            data.gsEnable = gsEnable;
            data.soEnable = soEnable;
            data.soTopology = soTopology;
            data.splitId = splitId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthInfoPixelRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyDepthInfoPixelRateData
    {
        // Fields
        uint64_t depthPassCount;
        uint64_t activeLanes;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthInfoPixelRate
    //////////////////////////////////////////////////////////////////////////
    struct EarlyDepthInfoPixelRate : Event
    {
        const uint32_t eventId = {52};
        EarlyDepthInfoPixelRateData data;

        // Constructor
        EarlyDepthInfoPixelRate(
            uint64_t depthPassCount,
            uint64_t activeLanes
        )
        {
            data.depthPassCount = depthPassCount;
            data.activeLanes = activeLanes;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoNullPSData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyDepthStencilInfoNullPSData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoNullPS
    //////////////////////////////////////////////////////////////////////////
    struct EarlyDepthStencilInfoNullPS : Event
    {
        const uint32_t eventId = {48};
        EarlyDepthStencilInfoNullPSData data;

        // Constructor
        EarlyDepthStencilInfoNullPS(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyDepthStencilInfoSampleRateData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct EarlyDepthStencilInfoSampleRate : Event
    {
        const uint32_t eventId = {47};
        EarlyDepthStencilInfoSampleRateData data;

        // Constructor
        EarlyDepthStencilInfoSampleRate(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyDepthStencilInfoSingleSampleData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyDepthStencilInfoSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct EarlyDepthStencilInfoSingleSample : Event
    {
        const uint32_t eventId = {46};
        EarlyDepthStencilInfoSingleSampleData data;

        // Constructor
        EarlyDepthStencilInfoSingleSample(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyOmStencilData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyOmStencilData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyOmStencil
    //////////////////////////////////////////////////////////////////////////
    struct EarlyOmStencil : Event
    {
        const uint32_t eventId = {29};
        EarlyOmStencilData data;

        // Constructor
        EarlyOmStencil(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyOmZData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyOmZData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyOmZ
    //////////////////////////////////////////////////////////////////////////
    struct EarlyOmZ : Event
    {
        const uint32_t eventId = {28};
        EarlyOmZData data;

        // Constructor
        EarlyOmZ(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyStencilData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencil
    //////////////////////////////////////////////////////////////////////////
    struct EarlyStencil : Event
    {
        const uint32_t eventId = {22};
        EarlyStencilData data;

        // Constructor
        EarlyStencil(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilNullPSData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyStencilNullPSData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilNullPS
    //////////////////////////////////////////////////////////////////////////
    struct EarlyStencilNullPS : Event
    {
        const uint32_t eventId = {25};
        EarlyStencilNullPSData data;

        // Constructor
        EarlyStencilNullPS(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyStencilSampleRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct EarlyStencilSampleRate : Event
    {
        const uint32_t eventId = {18};
        EarlyStencilSampleRateData data;

        // Constructor
        EarlyStencilSampleRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyStencilSingleSampleData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyStencilSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct EarlyStencilSingleSample : Event
    {
        const uint32_t eventId = {14};
        EarlyStencilSingleSampleData data;

        // Constructor
        EarlyStencilSingleSample(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyZData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZ
    //////////////////////////////////////////////////////////////////////////
    struct EarlyZ : Event
    {
        const uint32_t eventId = {20};
        EarlyZData data;

        // Constructor
        EarlyZ(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZNullPSData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyZNullPSData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZNullPS
    //////////////////////////////////////////////////////////////////////////
    struct EarlyZNullPS : Event
    {
        const uint32_t eventId = {24};
        EarlyZNullPSData data;

        // Constructor
        EarlyZNullPS(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZPixelRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyZPixelRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZPixelRate
    //////////////////////////////////////////////////////////////////////////
    struct EarlyZPixelRate : Event
    {
        const uint32_t eventId = {26};
        EarlyZPixelRateData data;

        // Constructor
        EarlyZPixelRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyZSampleRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct EarlyZSampleRate : Event
    {
        const uint32_t eventId = {16};
        EarlyZSampleRateData data;

        // Constructor
        EarlyZSampleRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct EarlyZSingleSampleData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::EarlyZSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct EarlyZSingleSample : Event
    {
        const uint32_t eventId = {12};
        EarlyZSingleSampleData data;

        // Constructor
        EarlyZSingleSample(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::FrontendDrawEndEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct FrontendDrawEndEventData
    {
        // Fields
        uint32_t drawId;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::FrontendDrawEndEvent
    //////////////////////////////////////////////////////////////////////////
    struct FrontendDrawEndEvent : Event
    {
        const uint32_t eventId = {55};
        FrontendDrawEndEventData data;

        // Constructor
        FrontendDrawEndEvent(
            uint32_t drawId
        )
        {
            data.drawId = drawId;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::FrontendStatsEventData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct FrontendStatsEventData
    {
        // Fields
        uint32_t drawId;
        uint64_t IaVertices;
        uint64_t IaPrimitives;
        uint64_t VsInvocations;
        uint64_t HsInvocations;
        uint64_t DsInvocations;
        uint64_t GsInvocations;
        uint64_t GsPrimitives;
        uint64_t CInvocations;
        uint64_t CPrimitives;
        uint64_t SoPrimStorageNeeded0;
        uint64_t SoPrimStorageNeeded1;
        uint64_t SoPrimStorageNeeded2;
        uint64_t SoPrimStorageNeeded3;
        uint64_t SoNumPrimsWritten0;
        uint64_t SoNumPrimsWritten1;
        uint64_t SoNumPrimsWritten2;
        uint64_t SoNumPrimsWritten3;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::FrontendStatsEvent
    //////////////////////////////////////////////////////////////////////////
    struct FrontendStatsEvent : Event
    {
        const uint32_t eventId = {10};
        FrontendStatsEventData data;

        // Constructor
        FrontendStatsEvent(
            uint32_t drawId,
            uint64_t IaVertices,
            uint64_t IaPrimitives,
            uint64_t VsInvocations,
            uint64_t HsInvocations,
            uint64_t DsInvocations,
            uint64_t GsInvocations,
            uint64_t GsPrimitives,
            uint64_t CInvocations,
            uint64_t CPrimitives,
            uint64_t SoPrimStorageNeeded0,
            uint64_t SoPrimStorageNeeded1,
            uint64_t SoPrimStorageNeeded2,
            uint64_t SoPrimStorageNeeded3,
            uint64_t SoNumPrimsWritten0,
            uint64_t SoNumPrimsWritten1,
            uint64_t SoNumPrimsWritten2,
            uint64_t SoNumPrimsWritten3
        )
        {
            data.drawId = drawId;
            data.IaVertices = IaVertices;
            data.IaPrimitives = IaPrimitives;
            data.VsInvocations = VsInvocations;
            data.HsInvocations = HsInvocations;
            data.DsInvocations = DsInvocations;
            data.GsInvocations = GsInvocations;
            data.GsPrimitives = GsPrimitives;
            data.CInvocations = CInvocations;
            data.CPrimitives = CPrimitives;
            data.SoPrimStorageNeeded0 = SoPrimStorageNeeded0;
            data.SoPrimStorageNeeded1 = SoPrimStorageNeeded1;
            data.SoPrimStorageNeeded2 = SoPrimStorageNeeded2;
            data.SoPrimStorageNeeded3 = SoPrimStorageNeeded3;
            data.SoNumPrimsWritten0 = SoNumPrimsWritten0;
            data.SoNumPrimsWritten1 = SoNumPrimsWritten1;
            data.SoNumPrimsWritten2 = SoNumPrimsWritten2;
            data.SoNumPrimsWritten3 = SoNumPrimsWritten3;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSInputPrimsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSInputPrimsData
    {
        // Fields
        uint32_t drawId;
        uint64_t inputPrimCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSInputPrims
    //////////////////////////////////////////////////////////////////////////
    struct GSInputPrims : Event
    {
        const uint32_t eventId = {32};
        GSInputPrimsData data;

        // Constructor
        GSInputPrims(
            uint32_t drawId,
            uint64_t inputPrimCount
        )
        {
            data.drawId = drawId;
            data.inputPrimCount = inputPrimCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSPrimInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSPrimInfoData
    {
        // Fields
        uint64_t inputPrimCount;
        uint64_t primGeneratedCount;
        uint64_t vertsInput;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSPrimInfo
    //////////////////////////////////////////////////////////////////////////
    struct GSPrimInfo : Event
    {
        const uint32_t eventId = {60};
        GSPrimInfoData data;

        // Constructor
        GSPrimInfo(
            uint64_t inputPrimCount,
            uint64_t primGeneratedCount,
            uint64_t vertsInput
        )
        {
            data.inputPrimCount = inputPrimCount;
            data.primGeneratedCount = primGeneratedCount;
            data.vertsInput = vertsInput;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSPrimsGenData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSPrimsGenData
    {
        // Fields
        uint32_t drawId;
        uint64_t primGeneratedCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSPrimsGen
    //////////////////////////////////////////////////////////////////////////
    struct GSPrimsGen : Event
    {
        const uint32_t eventId = {33};
        GSPrimsGenData data;

        // Constructor
        GSPrimsGen(
            uint32_t drawId,
            uint64_t primGeneratedCount
        )
        {
            data.drawId = drawId;
            data.primGeneratedCount = primGeneratedCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSVertsInputData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSVertsInputData
    {
        // Fields
        uint32_t drawId;
        uint64_t vertsInput;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::GSVertsInput
    //////////////////////////////////////////////////////////////////////////
    struct GSVertsInput : Event
    {
        const uint32_t eventId = {34};
        GSVertsInputData data;

        // Constructor
        GSVertsInput(
            uint32_t drawId,
            uint64_t vertsInput
        )
        {
            data.drawId = drawId;
            data.vertsInput = vertsInput;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthInfoPixelRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateDepthInfoPixelRateData
    {
        // Fields
        uint64_t depthPassCount;
        uint64_t activeLanes;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthInfoPixelRate
    //////////////////////////////////////////////////////////////////////////
    struct LateDepthInfoPixelRate : Event
    {
        const uint32_t eventId = {53};
        LateDepthInfoPixelRateData data;

        // Constructor
        LateDepthInfoPixelRate(
            uint64_t depthPassCount,
            uint64_t activeLanes
        )
        {
            data.depthPassCount = depthPassCount;
            data.activeLanes = activeLanes;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoNullPSData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateDepthStencilInfoNullPSData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoNullPS
    //////////////////////////////////////////////////////////////////////////
    struct LateDepthStencilInfoNullPS : Event
    {
        const uint32_t eventId = {51};
        LateDepthStencilInfoNullPSData data;

        // Constructor
        LateDepthStencilInfoNullPS(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateDepthStencilInfoSampleRateData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct LateDepthStencilInfoSampleRate : Event
    {
        const uint32_t eventId = {50};
        LateDepthStencilInfoSampleRateData data;

        // Constructor
        LateDepthStencilInfoSampleRate(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateDepthStencilInfoSingleSampleData
    {
        // Fields
        uint64_t depthPassMask;
        uint64_t stencilPassMask;
        uint64_t coverageMask;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateDepthStencilInfoSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct LateDepthStencilInfoSingleSample : Event
    {
        const uint32_t eventId = {49};
        LateDepthStencilInfoSingleSampleData data;

        // Constructor
        LateDepthStencilInfoSingleSample(
            uint64_t depthPassMask,
            uint64_t stencilPassMask,
            uint64_t coverageMask
        )
        {
            data.depthPassMask = depthPassMask;
            data.stencilPassMask = stencilPassMask;
            data.coverageMask = coverageMask;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateOmStencilData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateOmStencilData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateOmStencil
    //////////////////////////////////////////////////////////////////////////
    struct LateOmStencil : Event
    {
        const uint32_t eventId = {31};
        LateOmStencilData data;

        // Constructor
        LateOmStencil(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateOmZData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateOmZData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateOmZ
    //////////////////////////////////////////////////////////////////////////
    struct LateOmZ : Event
    {
        const uint32_t eventId = {30};
        LateOmZData data;

        // Constructor
        LateOmZ(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencilData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateStencilData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencil
    //////////////////////////////////////////////////////////////////////////
    struct LateStencil : Event
    {
        const uint32_t eventId = {23};
        LateStencilData data;

        // Constructor
        LateStencil(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencilSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateStencilSampleRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencilSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct LateStencilSampleRate : Event
    {
        const uint32_t eventId = {19};
        LateStencilSampleRateData data;

        // Constructor
        LateStencilSampleRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencilSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateStencilSingleSampleData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateStencilSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct LateStencilSingleSample : Event
    {
        const uint32_t eventId = {15};
        LateStencilSingleSampleData data;

        // Constructor
        LateStencilSingleSample(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateZData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZ
    //////////////////////////////////////////////////////////////////////////
    struct LateZ : Event
    {
        const uint32_t eventId = {21};
        LateZData data;

        // Constructor
        LateZ(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZPixelRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateZPixelRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZPixelRate
    //////////////////////////////////////////////////////////////////////////
    struct LateZPixelRate : Event
    {
        const uint32_t eventId = {27};
        LateZPixelRateData data;

        // Constructor
        LateZPixelRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZSampleRateData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateZSampleRateData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZSampleRate
    //////////////////////////////////////////////////////////////////////////
    struct LateZSampleRate : Event
    {
        const uint32_t eventId = {17};
        LateZSampleRateData data;

        // Constructor
        LateZSampleRate(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZSingleSampleData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct LateZSingleSampleData
    {
        // Fields
        uint32_t drawId;
        uint64_t passCount;
        uint64_t failCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::LateZSingleSample
    //////////////////////////////////////////////////////////////////////////
    struct LateZSingleSample : Event
    {
        const uint32_t eventId = {13};
        LateZSingleSampleData data;

        // Constructor
        LateZSingleSample(
            uint32_t drawId,
            uint64_t passCount,
            uint64_t failCount
        )
        {
            data.drawId = drawId;
            data.passCount = passCount;
            data.failCount = failCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::RasterTileCountData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct RasterTileCountData
    {
        // Fields
        uint32_t drawId;
        uint64_t rasterTiles;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::RasterTileCount
    //////////////////////////////////////////////////////////////////////////
    struct RasterTileCount : Event
    {
        const uint32_t eventId = {59};
        RasterTileCountData data;

        // Constructor
        RasterTileCount(
            uint32_t drawId,
            uint64_t rasterTiles
        )
        {
            data.drawId = drawId;
            data.rasterTiles = rasterTiles;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::RasterTilesData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct RasterTilesData
    {
        // Fields
        uint32_t drawId;
        uint32_t rastTileCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::RasterTiles
    //////////////////////////////////////////////////////////////////////////
    struct RasterTiles : Event
    {
        const uint32_t eventId = {36};
        RasterTilesData data;

        // Constructor
        RasterTiles(
            uint32_t drawId,
            uint32_t rastTileCount
        )
        {
            data.drawId = drawId;
            data.rastTileCount = rastTileCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::TessPrimCountData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct TessPrimCountData
    {
        // Fields
        uint64_t primCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::TessPrimCount
    //////////////////////////////////////////////////////////////////////////
    struct TessPrimCount : Event
    {
        const uint32_t eventId = {58};
        TessPrimCountData data;

        // Constructor
        TessPrimCount(
            uint64_t primCount
        )
        {
            data.primCount = primCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::TessPrimsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct TessPrimsData
    {
        // Fields
        uint32_t drawId;
        uint64_t primCount;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// PipelineStats::TessPrims
    //////////////////////////////////////////////////////////////////////////
    struct TessPrims : Event
    {
        const uint32_t eventId = {35};
        TessPrimsData data;

        // Constructor
        TessPrims(
            uint32_t drawId,
            uint64_t primCount
        )
        {
            data.drawId = drawId;
            data.primCount = primCount;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_PIPELINE_STATS && IsEventEnabled;
        }
    };



    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::CSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct CSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::CSInfo
    //////////////////////////////////////////////////////////////////////////
    struct CSInfo : Event
    {
        const uint32_t eventId = {45};
        CSInfoData data;

        // Constructor
        CSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::CSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct CSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::CSStats
    //////////////////////////////////////////////////////////////////////////
    struct CSStats : Event
    {
        const uint32_t eventId = {71};
        CSStatsData data;

        // Constructor
        CSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::DSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::DSInfo
    //////////////////////////////////////////////////////////////////////////
    struct DSInfo : Event
    {
        const uint32_t eventId = {42};
        DSInfoData data;

        // Constructor
        DSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::DSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct DSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::DSStats
    //////////////////////////////////////////////////////////////////////////
    struct DSStats : Event
    {
        const uint32_t eventId = {68};
        DSStatsData data;

        // Constructor
        DSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::GSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::GSInfo
    //////////////////////////////////////////////////////////////////////////
    struct GSInfo : Event
    {
        const uint32_t eventId = {43};
        GSInfoData data;

        // Constructor
        GSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::GSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct GSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::GSStats
    //////////////////////////////////////////////////////////////////////////
    struct GSStats : Event
    {
        const uint32_t eventId = {69};
        GSStatsData data;

        // Constructor
        GSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::HSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct HSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::HSInfo
    //////////////////////////////////////////////////////////////////////////
    struct HSInfo : Event
    {
        const uint32_t eventId = {41};
        HSInfoData data;

        // Constructor
        HSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::HSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct HSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::HSStats
    //////////////////////////////////////////////////////////////////////////
    struct HSStats : Event
    {
        const uint32_t eventId = {67};
        HSStatsData data;

        // Constructor
        HSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::PSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct PSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::PSInfo
    //////////////////////////////////////////////////////////////////////////
    struct PSInfo : Event
    {
        const uint32_t eventId = {44};
        PSInfoData data;

        // Constructor
        PSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::PSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct PSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::PSStats
    //////////////////////////////////////////////////////////////////////////
    struct PSStats : Event
    {
        const uint32_t eventId = {70};
        PSStatsData data;

        // Constructor
        PSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::VSInfoData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct VSInfoData
    {
        // Fields
        uint32_t drawId;
        uint32_t numInstExecuted;
        uint32_t numSampleExecuted;
        uint32_t numSampleLExecuted;
        uint32_t numSampleBExecuted;
        uint32_t numSampleCExecuted;
        uint32_t numSampleCLZExecuted;
        uint32_t numSampleCDExecuted;
        uint32_t numGather4Executed;
        uint32_t numGather4CExecuted;
        uint32_t numGather4CPOExecuted;
        uint32_t numGather4CPOCExecuted;
        uint32_t numLodExecuted;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::VSInfo
    //////////////////////////////////////////////////////////////////////////
    struct VSInfo : Event
    {
        const uint32_t eventId = {40};
        VSInfoData data;

        // Constructor
        VSInfo(
            uint32_t drawId,
            uint32_t numInstExecuted,
            uint32_t numSampleExecuted,
            uint32_t numSampleLExecuted,
            uint32_t numSampleBExecuted,
            uint32_t numSampleCExecuted,
            uint32_t numSampleCLZExecuted,
            uint32_t numSampleCDExecuted,
            uint32_t numGather4Executed,
            uint32_t numGather4CExecuted,
            uint32_t numGather4CPOExecuted,
            uint32_t numGather4CPOCExecuted,
            uint32_t numLodExecuted
        )
        {
            data.drawId = drawId;
            data.numInstExecuted = numInstExecuted;
            data.numSampleExecuted = numSampleExecuted;
            data.numSampleLExecuted = numSampleLExecuted;
            data.numSampleBExecuted = numSampleBExecuted;
            data.numSampleCExecuted = numSampleCExecuted;
            data.numSampleCLZExecuted = numSampleCLZExecuted;
            data.numSampleCDExecuted = numSampleCDExecuted;
            data.numGather4Executed = numGather4Executed;
            data.numGather4CExecuted = numGather4CExecuted;
            data.numGather4CPOExecuted = numGather4CPOExecuted;
            data.numGather4CPOCExecuted = numGather4CPOCExecuted;
            data.numLodExecuted = numLodExecuted;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::VSStatsData
    //////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
    struct VSStatsData
    {
        // Fields
        HANDLE hStats;
    };
#pragma pack(pop)

    //////////////////////////////////////////////////////////////////////////
    /// ShaderStats::VSStats
    //////////////////////////////////////////////////////////////////////////
    struct VSStats : Event
    {
        const uint32_t eventId = {66};
        VSStatsData data;

        // Constructor
        VSStats(
            HANDLE hStats
        )
        {
            data.hStats = hStats;
        }

        virtual void Accept(EventHandler* pHandler) const;
        inline const uint32_t GetEventId() const { return eventId; }
        
        bool IsEnabled() const
        {
            static const bool IsEventEnabled = true;    // TODO: Replace with knob for each event
            return KNOB_AR_ENABLE_SHADER_STATS && IsEventEnabled;
        }
    };


} // namespace ArchRast
// clang-format on
